# !/usr/bin/env python
# encoding: utf-8
import json

from .headers import *
from email.feedparser import FeedParser


BOUNDARY_SEPARATOR = '--'
BODY_SEPARATOR = "\n\n"
UNAUTHORIZED_STATUS = 401


class Response(Headers):
    def __init__(self, status, raw, headers=None):
        """
         Builds response by parsing raw response on-the-wire. Should be refactored out. Parsing should not happen here.
         It's not the place for this.
        :param status:
        :param raw:
        :param headers:
        :return:
        """
        Headers.__init__(self)

        self.__body = ''
        self.__raw = raw.replace('\r', '')
        self.__raw_headers = ''
        self.__data = None
        self.__status = status
        self.__responses = []

        if status is None:
            raise Exception('Empty status was received')

        if isinstance(headers, dict):
            self.set_headers(headers)
            self.__body = raw
            self.__parse_headers()

        elif self.__raw.find(BODY_SEPARATOR) > 0:
            self.__raw_headers, self.__body = self.__raw.split(BODY_SEPARATOR, 1)

        else:
            self.__body = self.__raw

        self.__parse_body()

    def check_status(self):
        """
        Returns True if HTTP status is 200 OK
        :return:
        """
        return 200 <= self.__status < 300

    def get_body(self):
        """
        Returns whole response body as a string.
        :return:
        """
        return self.__body

    def get_data(self):
        """
        Returns:
         - None for multipart response,
         - JSON for application/json response,
         - or body for anything else
        """
        return self.__data

    def get_raw(self):
        """
        Raw response with headers and body. As seen on the wire. (WTF?!)
        :return:
        """
        return self.__raw

    def get_status(self):
        """
        Returns HTTP status code
        :return:
        """
        return self.__status

    def get_responses(self):
        """
        Returns sub-responses, if this response is multipart
        :return:
        """
        return self.__responses

    def __parse_headers(self):
        """
        Sic! HTTP clients do this!
        :return:
        """
        headers = self.__raw_headers.split('\n')
        for h in filter(lambda x: x.find(HEADER_SEPARATOR) >= 0, headers):
            k, v = h.split(HEADER_SEPARATOR, 1)
            self.set_header(k.strip(), v.strip())

    def __break_into_parts(self):

        p = FeedParser()
        for h in self.get_headers_array():
            p.feed(h + "\r\n")
        p.feed("\r\n")
        p.feed(self.__body)
        msg = p.close()
        parts = msg.get_payload()
        return parts

    def __parse_body(self):
        """
        Parses body (facepalm)
        """
        if self.is_multipart():
            parts = self.__break_into_parts()
            if len(parts) < 1:
                # sic! not specific extension
                raise Exception("Malformed Batch Response (not enough parts)")
            # todo: defensive checking for part[0] content type
            # todo: JSON parsing errors handling
            statuses = json.loads(parts[0].get_payload())
            if len(statuses["response"]) != len(parts) - 1:
                raise Exception("Malformed Batch Response (not-consistent number of parts)")

            for response, payload in zip(statuses["response"], parts[1:]):
                self.__responses.append(Response(response["status"], payload.get_payload(), dict(payload)))

        elif self.is_json():
            self.__data = json.loads(self.__body)
        else:
            self.__data = self.__body



#!/usr/bin/env python
# encoding: utf-8

import os
import json

from .cache import Cache


class FileCache(Cache):
    def __init__(self, cache_dir):
        Cache.__init__(self)
        self.cache_dir = cache_dir

    def save(self, key, obj):
        if not os.path.isdir(self.cache_dir):
            os.mkdir(self.cache_dir)
        f = open(self._file_path(key + '.json'), 'w')
        json.dump(obj, f, indent=4)
        f.close()
        Cache.save(self, key, obj)

    def load(self, key):
        try:
            f = open(self._file_path(key + '.json'), 'r')
            data = json.load(f)
            f.close()
            return data if data else {}
        except IOError:
            # This is just a cache, we don't care about errors.
            return {}

    def _file_path(self, name):
        return os.path.join(self.cache_dir, name)


import httplib
from urlparse import urlparse
from .response import Response
from .ajax_exception import AjaxException

class Ajax:
    def __init__(self, request):
        self.__request = request
        self.__response = None

    def send(self):

        url = urlparse(self.__request.get_url())
        if url.scheme == "https":
            conn = httplib.HTTPSConnection(url.hostname, url.port)
        else:
            conn = httplib.HTTPConnection(url.hostname, url.port)
        try:
            conn.request(self.__request.get_method(),
                         self.__request.get_url(),
                         body=self.__request.get_encoded_body(),
                         headers=self.__request.get_headers())

            response = conn.getresponse()
            body = response.read()
            status_code = response.status
            headers = dict(response.getheaders())

            self.__response = Response(status_code, body, dict(headers))
            if not self.__response.check_status():
                raise AjaxException(self)

        finally:
            conn.close()

    def is_loaded(self):
        return True if self.__response else False

    def get_response(self):
        return self.__response

    def get_request(self):
        return self.__request


#!/usr/bin/env python
# encoding: utf-8

import base64
import time

from .auth import Auth
from rcsdk.core.ajax.request import *
from rcsdk.core.ajax.ajax import Ajax

ACCOUNT_ID = '~'
ACCOUNT_PREFIX = '/account/'
URL_PREFIX = '/restapi'
TOKEN_ENDPOINT = '/restapi/oauth/token'
API_VERSION = 'v1.0'
ACCESS_TOKEN_TTL = 600  # 10 minutes
REFRESH_TOKEN_TTL = 36000  # 10 hours
REFRESH_TOKEN_TTL_REMEMBER = 604800  # 1 week
SERVER = 'https://platform.ringcentral.com'


class Platform:
    def __init__(self, cache, key, secret, server=None):
        self.__server = server if server else SERVER
        self.appKey = key
        self.appSecret = secret
        self.__auth = Auth(cache)
        self.__account = ACCOUNT_ID

    def is_authorized(self, refresh=True):
        if not self.__auth.is_access_token_valid() and refresh:
            self.refresh()
        if not self.__auth.is_access_token_valid():
            raise Exception('Access token is not valid after refresh timeout')

    def authorize(self, user_name, extension, password, remember=False):
        ajax = self.auth_call(Request(POST, TOKEN_ENDPOINT, body={
            'grant_type': 'password',
            'username': user_name,
            'extension': extension,
            'password': password,
            'access_toket_ttl': ACCESS_TOKEN_TTL,
            'refresh_token_ttl': REFRESH_TOKEN_TTL_REMEMBER if remember else REFRESH_TOKEN_TTL
        }))
        self.__auth.set_data(ajax.get_response().get_data())
        self.__auth.set_remember(remember)

    def refresh(self):
        if not self.__auth.is_paused():

            print("Refresh will be performed\n")
            self.__auth.pause()

            if not self.__auth.is_refresh_token_valid():
                raise Exception('Refresh token has expired')

            ajax = self.auth_call(Request(POST, TOKEN_ENDPOINT, body={
                'grant_type': 'refresh_token',
                'refresh_token': self.__auth.get_refresh_token(),
                'access_token_ttl': ACCESS_TOKEN_TTL,
                'refresh_token_ttl': REFRESH_TOKEN_TTL_REMEMBER if self.__auth.is_remember() else REFRESH_TOKEN_TTL
            }))

            self.__auth.set_data(ajax.get_response().get_data())

            self.__auth.resume()

        else:

            while self.__auth.is_paused():
                print("Waiting for refresh\n")
                time.sleep(1)
            self.is_authorized(False)

    def api_call(self, request):
        self.is_authorized()
        request.set_header(AUTHORIZATION, self.__get_auth_header())
        request.set_url(self.api_url(request.get_url(), {'addServer': True}))
        ajax = Ajax(request)
        ajax.send()
        return ajax

    def auth_call(self, request):
        request.set_header(AUTHORIZATION, 'Basic ' + self.__get_api_key())
        request.set_header(CONTENT_TYPE, URL_ENCODED_CONTENT_TYPE)
        request.set_url(self.api_url(request.get_url(), {'addServer': True}))
        request.set_method(POST)
        ajax = Ajax(request)
        ajax.send()
        return ajax

    def __get_api_key(self):
        return base64.b64encode(self.appKey + ':' + self.appSecret)

    def __get_auth_header(self):
        return self.__auth.get_token_type() + ' ' + self.__auth.get_access_token()

    def api_url(self, url, options=None):
        built_url = ''
        options = options if options else {}

        if 'addServer' in options and options['addServer'] and url.find('http://') < 0 and url.find('https://') < 0:
            built_url += self.__server

        if url.find(URL_PREFIX) < 0:
            built_url += URL_PREFIX + '/' + API_VERSION

        if url.find(ACCOUNT_PREFIX) < 0:
            built_url = built_url.replace(ACCOUNT_PREFIX + ACCOUNT_ID, ACCOUNT_PREFIX + self.__account)

        built_url += url

        if 'addMethod' in options and options['addMethod']:
            built_url += ('&' if url.find('?') >= 0 else '?') + '_method=' + options['addMethod']

        if 'addToken' in options and options['addToken']:
            built_url += ('&' if url.find('?') >= 0 else '?') + 'access_token=' + options['addMethod']

        return built_url






#!/usr/bin/env python
# encoding: utf-8


class AjaxException(Exception):
    def __init__(self, ajax, prev_exception=None):
        self.__ajax = ajax
        response = ajax.get_response()
        data = response.get_data() if response else {}
        status = response.get_status() if response else 500
        message = ''

        if 'message' in data:
            message = data['message']
        elif 'error_description' in data:
            message = data['error_description']
        elif 'description' in data:
            message = data['description']

        if prev_exception:
            message = str(prev_exception)

        message = message or 'Unknown error'

        Exception.__init__(self, status, message)

    def get_ajax(self):
        return self.__ajax


#!/usr/bin/env python
# encoding: utf-8

from .cache import Cache


class MemoryCache(Cache):
    def __init__(self):
        Cache.__init__(self)
        self.store = {}

    def save(self, key, obj):
        self.store[key] = obj
        Cache.save(self, key, obj)

    def load(self, key):
        return self.store.get(key, {})


#!/usr/bin/env python
# encoding: utf-8

from time import time


RELEASE_TIMEOUT = 10
CACHE_KEY = 'platform'

DEFAULT_AUTH_DATA = {
    'paused_time': 0,
    'token_type': '',
    'access_token': '',
    'expires_in': 0,
    'expire_time': 0,
    'refresh_token': '',
    'refresh_token_expires_in': 0
}


class Auth:
    def __init__(self, cache):
        self._cache = cache

    def set_data(self, auth_data=None):
        merged_data = dict(DEFAULT_AUTH_DATA.items())
        merged_data.update(self.get_data())
        merged_data.update(auth_data)

        if auth_data.get('expires_in') and not auth_data.get('expire_time'):
            merged_data['expire_time'] = time() + auth_data.get('expires_in')

        if auth_data.get('refresh_token_expires_in') and not auth_data.get('refresh_token_expire_time'):
            merged_data['refresh_token_expire_time'] = time() + auth_data.get('refresh_token_expires_in')

        self._cache.save(CACHE_KEY, merged_data)

    def get_data(self):
        cached = self._cache.load(CACHE_KEY)
        return cached if cached else {}

    def get_access_token(self):
        return self.get_data().get('access_token')

    def get_refresh_token(self):
        return self.get_data().get('refresh_token')

    def get_token_type(self):
        return self.get_data().get('token_type')

    def is_access_token_valid(self):
        return self.__is_token_date_valid(self.get_data().get('expire_time'))

    def is_refresh_token_valid(self):
        return self.__is_token_date_valid(self.get_data().get('refresh_token_expire_time'))

    def is_paused(self):
        pt = self.get_data().get('paused_time')
        return pt > 0 and (time() - pt) < RELEASE_TIMEOUT

    def pause(self):
        self.set_data({'paused_time': time()})

    def resume(self):
        self.set_data({'paused_time': 0})

    def set_remember(self, val):
        self.set_data({'remember': True if val else False})

    def is_remember(self):
        return self.get_data().get('remember', False)

    @staticmethod
    def __is_token_date_valid(token_date):
        return token_date > time()


#!/usr/bin/env python
# encoding: utf-8

VERSION = '0.2.3'


from core.platform.platform import Platform
from core.subscription.subscription import Subscription

class RCSDK:
    def __init__(self, cache, key, secret, server=''):
        self.__platform = Platform(cache, key, secret, server)

    def get_platform(self):
        return self.__platform

    def get_subscription(self):
        return Subscription(self.__platform)
